@@ -97,7 +97,7 @@ public:
                 AstActive* const activep = new AstActive{
                     flp, "force-init",
                     new AstSenTree{flp, new AstSenItem{flp, AstSenItem::Static{}}}};
-                activep->sensesStorep(activep->sensesp());
+                activep->senTreeStorep(activep->sentreep());
 
                 activep->addStmtsp(new AstInitial{flp, assignp});
                 vscp->scopep()->addBlocksp(activep);
@@ -120,7 +120,7 @@ public:
                     new AstSenItem{flp, VEdgeType::ET_CHANGED, origp->cloneTree(false)});
                 AstActive* const activep
                     = new AstActive{flp, "force-update", new AstSenTree{flp, itemsp}};
-                activep->sensesStorep(activep->sensesp());
+                activep->senTreeStorep(activep->sentreep());
                 activep->addStmtsp(new AstAlways{flp, VAlwaysKwd::ALWAYS, nullptr,
                                                  new AstAssign{flp, lhsp, rhsp}});
                 vscp->scopep()->addBlocksp(activep);
@@ -148,13 +148,16 @@ private:
     //  AstVar::user1p        -> ForceComponentsVar* instance (via m_forceComponentsVar)
     //  AstVarScope::user1p   -> ForceComponentsVarScope* instance (via m_forceComponentsVarScope)
     //  AstVarRef::user2      -> Flag indicating not to replace reference
-    //  AstVarScope::user3      -> AstVarScope*, a `valVscp` force component for each VarScope of
-    //  forced RHS
+    //  AstVarScope::user3p   -> AstNodeExpr*, the RHS expression
     const VNUser1InUse m_user1InUse;
     const VNUser2InUse m_user2InUse;
     const VNUser3InUse m_user3InUse;
     AstUser1Allocator<AstVar, ForceComponentsVar> m_forceComponentsVar;
     AstUser1Allocator<AstVarScope, ForceComponentsVarScope> m_forceComponentsVarScope;
+    std::unordered_map<const AstVarScope*,
+                       std::pair<std::unordered_set<AstVarScope*>, std::vector<AstVarScope*>>>
+        m_valVscps;
+    // `valVscp` force components of a forced RHS
 
 public:
     // CONSTRUCTORS
@@ -170,11 +173,21 @@ public:
     }
     static bool isNotReplaceable(const AstVarRef* const nodep) { return nodep->user2(); }
     static void markNonReplaceable(AstVarRef* const nodep) { nodep->user2SetOnce(); }
-    static AstVarScope* getValVscp(AstVarRef* const refp) {
-        return VN_CAST(refp->varScopep()->user3p(), VarScope);
+
+    // Get all ValVscps for a VarScope
+    const std::vector<AstVarScope*>* getValVscps(AstVarRef* const refp) const {
+        auto it = m_valVscps.find(refp->varScopep());
+        if (it != m_valVscps.end()) return &(it->second.second);
+        return nullptr;
     }
-    static void setValVscp(AstNodeVarRef* const refp, AstVarScope* const vscp) {
-        refp->varScopep()->user3p(vscp);
+
+    // Add a ValVscp for a VarScope
+    void addValVscp(AstVarRef* const refp, AstVarScope* const valVscp) {
+        if (m_valVscps[refp->varScopep()].first.find(valVscp)
+            != m_valVscps[refp->varScopep()].first.end())
+            return;
+        m_valVscps[refp->varScopep()].first.emplace(valVscp);
+        m_valVscps[refp->varScopep()].second.push_back(valVscp);
     }
 
     // METHODS
@@ -185,6 +198,12 @@ public:
     ForceComponentsVarScope* tryGetForceComponents(AstVarRef* nodep) const {
         return m_forceComponentsVarScope.tryGet(nodep->varScopep());
     }
+    void setValVscpRhsExpr(AstVarScope* valVscp, AstNodeExpr* rhsExpr) {
+        valVscp->user3p(rhsExpr);
+    }
+    AstNodeExpr* getValVscpRhsExpr(AstVarScope* valVscp) const {
+        return VN_CAST(valVscp->user3p(), NodeExpr);
+    }
 };
 
 class ForceConvertVisitor final : public VNVisitor {
@@ -233,9 +252,8 @@ class ForceConvertVisitor final : public VNVisitor {
             = new AstAssign{flp, lhsp->cloneTreePure(false), rhsp->cloneTreePure(false)};
         transformWritenVarScopes(setValp->lhsp(), [this, rhsp](AstVarScope* vscp) {
             AstVarScope* const valVscp = m_state.getForceComponents(vscp).m_valVscp;
-            // TODO support multiple VarRefs on RHS
-            if (AstVarRef* const refp = VN_CAST(rhsp, VarRef))
-                ForceState::setValVscp(refp, valVscp);
+            m_state.setValVscpRhsExpr(valVscp, rhsp->cloneTreePure(false));
+            rhsp->foreach([valVscp, this](AstVarRef* refp) { m_state.addValVscp(refp, valVscp); });
             return valVscp;
         });
 
@@ -267,15 +285,16 @@ class ForceConvertVisitor final : public VNVisitor {
         transformWritenVarScopes(resetEnp->lhsp(), [this](AstVarScope* vscp) {
             return m_state.getForceComponents(vscp).m_enVscp;
         });
-        // IEEE 1800-2023 10.6.2: If this is a net, and not a variable, then reset the read
-        // signal directly as well, in case something in the same process reads it later. Also, if
-        // it is a variable, and not a net, set the original signal to the forced value, as it
-        // needs to retain the forced value until the next procedural update, which might happen on
-        // a later eval. Luckily we can do all this in a single assignment.
-        FileLine* const fl_nowarn = new FileLine{flp};
-        fl_nowarn->warnOff(V3ErrorCode::BLKANDNBLK, true);
+
+        // IEEE 1800-2023 10.6.2: When released, then if the variable is not driven by a continuous
+        // assignment and does not currently have an active procedural continuous assignment, the
+        // variable shall not immediately change value and shall maintain its current value until
+        // the next procedural assignment to the variable is executed. Releasing a variable that is
+        // driven by a continuous assignment or currently has an active assign procedural
+        // continuous assignment shall reestablish that assignment and schedule a reevaluation in
+        // the continuous assignment's scheduling region.
         AstAssign* const resetRdp
-            = new AstAssign{fl_nowarn, lhsp->cloneTreePure(false), lhsp->unlinkFrBack()};
+            = new AstAssign{flp, lhsp->cloneTreePure(false), lhsp->unlinkFrBack()};
         // Replace write refs on the LHS
         resetRdp->lhsp()->foreach([this](AstNodeVarRef* refp) {
             if (refp->access() != VAccess::WRITE) return;
@@ -283,10 +302,7 @@ class ForceConvertVisitor final : public VNVisitor {
             AstVarScope* const newVscp = vscp->varp()->isContinuously()
                                              ? m_state.getForceComponents(vscp).m_rdVscp
                                              : vscp;
-            // Disable BLKANDNBLK for this reference
-            FileLine* const flp = new FileLine{refp->fileline()};
-            flp->warnOff(V3ErrorCode::BLKANDNBLK, true);
-            AstVarRef* const newpRefp = new AstVarRef{flp, newVscp, VAccess::WRITE};
+            AstVarRef* const newpRefp = new AstVarRef{refp->fileline(), newVscp, VAccess::WRITE};
             refp->replaceWith(newpRefp);
             VL_DO_DANGLING(refp->deleteTree(), refp);
         });
@@ -320,7 +336,8 @@ class ForceConvertVisitor final : public VNVisitor {
 
 public:
     // CONSTRUCTOR
-    explicit ForceConvertVisitor(AstNetlist* nodep, ForceState& state)
+    // cppcheck-suppress constParameterCallback
+    ForceConvertVisitor(AstNetlist* nodep, ForceState& state)
         : m_state{state} {
         // Transform all force and release statements
         iterateAndNextNull(nodep->modulesp());
@@ -359,9 +376,6 @@ class ForceReplaceVisitor final : public VNVisitor {
             // Replace VarRef from forced LHS with rdVscp.
             if (ForceState::ForceComponentsVarScope* const fcp
                 = m_state.tryGetForceComponents(nodep)) {
-                FileLine* const flp = nodep->fileline();
-                AstVarRef* const origp = new AstVarRef{flp, nodep->varScopep(), VAccess::READ};
-                ForceState::markNonReplaceable(origp);
                 nodep->varp(fcp->m_rdVscp->varp());
                 nodep->varScopep(fcp->m_rdVscp);
             }
@@ -378,20 +392,27 @@ class ForceReplaceVisitor final : public VNVisitor {
                 m_stmtp->addNextHere(new AstAssign{flp, lhsp, rhsp});
             }
             // Emit valVscp update after each write to any VarRef on forced RHS.
-            if (AstVarScope* const valVscp = ForceState::getValVscp(nodep)) {
+            if (!m_state.getValVscps(nodep)) break;
+            for (AstVarScope* const valVscp : *m_state.getValVscps(nodep)) {
                 FileLine* const flp = nodep->fileline();
                 AstVarRef* const valp = new AstVarRef{flp, valVscp, VAccess::WRITE};
-                AstVarRef* const rhsp = new AstVarRef{flp, nodep->varScopep(), VAccess::READ};
+                AstNodeExpr* rhsp = m_state.getValVscpRhsExpr(valVscp);
+                UASSERT_OBJ(rhsp, flp, "RHS of force/release must be an AstNodeExpr");
+                rhsp = rhsp->cloneTreePure(false);
 
                 ForceState::markNonReplaceable(valp);
-                ForceState::markNonReplaceable(rhsp);
+                rhsp->foreach([](AstVarRef* refp) { ForceState::markNonReplaceable(refp); });
 
                 m_stmtp->addNextHere(new AstAssign{flp, valp, rhsp});
             }
             break;
         }
         default:
-            nodep->v3error("Unsupported: Signals used via read-write reference cannot be forced");
+            if (!m_inLogic) return;
+            if (m_state.tryGetForceComponents(nodep) || m_state.getValVscps(nodep)) {
+                nodep->v3error(
+                    "Unsupported: Signals used via read-write reference cannot be forced");
+            }
             break;
         }
     }
@@ -408,7 +429,7 @@ public:
 //
 
 void V3Force::forceAll(AstNetlist* nodep) {
-    UINFO(2, __FUNCTION__ << ": " << endl);
+    UINFO(2, __FUNCTION__ << ":");
     if (!v3Global.hasForceableSignals()) return;
     {
         ForceState state;
